package com.sdforce.ws;

import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.xml.sax.InputSource;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
/**
 * 决策引擎接口返回消息对象类，返回对象包括如下几部分：
 * 接口调用状态字段 :status，状态正常返回 ok，有错误返回 fail
 * 状态消息:statusMsg，当状态为 ok时为空，状态为 fail时，为错误消息
 * 申请流水号：reqSerialNo,与调用时的输入流水号相同
 * 系统返回的序列号-此编号在调用决策引擎的记录中唯一
 * 具体的单条返回数据:在body内的resultParams标签内，返回具体的json串。
 * @author 7DForceA1
 *
 */
public class RspMsg {
	private String status;
	private String errMsg;
	private String reqSerialNo;//申请流水号
	private String sysSerialNo;//系统流水号
	
	

	private Map<String, String> rspData=new HashMap<String, String>();
	
	public String getStatus() {
		if(status==null) return "";
		return status;
	}
	public void setStatus(String status) {
		this.status = status;
	}
	public String getErrMsg() {
		return errMsg;
	}
	public void setErrMsg(String errMsg) {
		this.errMsg = errMsg;
	}
	
	
	public String getReqSerialNo() {
		return reqSerialNo;
	}
	public void setReqSerialNo(String reqSerialNo) {
		this.reqSerialNo = reqSerialNo;
	}
	public String getSysSerialNo() {
		return sysSerialNo;
	}
	public void setSysSerialNo(String sysSerialNo) {
		this.sysSerialNo = sysSerialNo;
	}
	public Map<String, String> getRspData() {
		return rspData;
	}
	public void setRspData(Map<String, String> rspData) {
		this.rspData = rspData;
	}
	
	/**
	 * 将返回的xml转换成对象
	 * @param rspStr
	 * @return
	 */
	public static RspMsg parseRspXML2RspObj(String rspStr) {
		SAXReader reader = new SAXReader();
        StringReader read = new StringReader(rspStr);  
        InputSource source = new InputSource(read);  		
		Document xmldoc;
		try {
			xmldoc = reader.read(source);
			Element root = xmldoc.getRootElement();
			Element body = root.element("Body");
			if(body==null){
				System.out.println("Parse XML failed: Body not find");
				return null;
			}
			Element decRsp= body.element("singleDecisionResponse");
			if(decRsp==null){
				System.out.println("Parse XML failed: singleDecisionResponse not find");				
				return null;
			}
			Element retNode=decRsp.element("return");
			if(retNode==null){
				System.out.println("Parse XML failed: return node not find");				
				return null;
			}
			
			//创建一个返回消息对象
			RspMsg rsp=new RspMsg();
			
			//获得申请流水号
			Element reqNo=retNode.element("reqSerialNo");
			if(reqNo==null){
				System.out.println("Parse XML failed: reqSerialNo not find");				
				return null;
			}
			rsp.setReqSerialNo(reqNo.getText());
			
			//获得系统流水号
			Element sysNo=retNode.element("sysSerialNo");
			if(sysNo==null){
				System.out.println("Parse XML failed: sysSerialNo not find");				
				return null;
			}
			rsp.setSysSerialNo(sysNo.getText());
			
			//获得状态码
			Element status=retNode.element("status");
			if(status==null){
				System.out.println("Parse XML failed: status not find");				
				return null;
			}
			rsp.setStatus(status.getText());			

			//当状态为 ok时，读取body信息
			if("ok".equals(rsp.getStatus())){
				
				Element returnBody=retNode.element("body");
				if(returnBody!=null){
					Element rspDataEle= returnBody.element("resultParams");
					if(rspDataEle!=null){
						String rspDataStr=rspDataEle.getText();

						Gson gson=new Gson();
						Map<String,String> rspData=gson.fromJson(rspDataStr, 
								new TypeToken<Map<String, String>>(){}.getType());
						rsp.setRspData(rspData);
					}
				}
			}else{ 
				//当状态码为 fail时，获取错误信息
				Element errMsg=retNode.element("statusMsg");
				if(errMsg!=null){
					rsp.setErrMsg(errMsg.getText());
				}
			}
			
			return rsp;
			
		} catch (DocumentException e) {
            e.printStackTrace();
            return null;
		}
	}

	@Override
	public String toString() {
		return "RspMsg [status=" + status + ", errMsg=" + errMsg + ", reqSerialNo="
				+ reqSerialNo+", sysSerialNo=" + sysSerialNo + ", rspData="	+ rspData + "]";
	}
}
