package com.sdforce.ws;


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

import com.google.gson.Gson;

/**
 * windseeker决策引擎调用接口演示程序
 * 
 * @author 7dforce
 *
 */

public class WSIntDemo {
	/*
	 * Main方法 
	 * 1. get interface connection conf 得到接口联系配置信息
	 * 2. prepare singleRecord 准备单条输入数据
	 * 3. get reqNO 得到调用请求编号，示例中为使用日期生成，
	 *    实际业务中可使用业务相关的唯一标识，后续当业务出现问题时，可使用此唯一标识来查找日志记录
	 * 4. invode the ws interface 调用接口
	 * 5. get the response 得到返回结果
	 * 
	 */
	public static void main(String[] args){
		Scanner scan = new Scanner(System.in);
		
		boolean loop=true;
		
		while(loop){
			System.out.println("请选择要进行的操作：（D 调用Demo策略, I 调用interfaceTest策略, N 退出）");
			String func=scan.next();
			if(func==null){
				break;
			}
			func=func.toUpperCase();
			
			if(func.equals("N")){
				loop=false;
				scan.close();
				break;
			}
			
			else if(func.equals("D")||func.equals("I")){
				callWSServer(func);
			}
			else{
				System.out.println("输入错误，请重新选择！");
			}
				
		}
		if(scan!=null){
			scan.close();
			scan=null;
		}
				
	}

	private static void callWSServer(String func) {
		System.out.println("**************Demo Start**************");
		
		WSConnConf connConf=null;
		if(func.equals("D")){
			connConf = WSConnConf.getConnConfDemo();
		}
		else if(func.equals("I")){
			connConf = WSConnConf.getConnConfIF();
		}
		else{
			return;
		}
		System.out.println("1.获取连接配置信息 Get interface connection Conf:\n\t "+connConf);

		Map<String, Object> singleRecord = prepareRecord();	
		System.out.println("2. 构造测试数据 The singleRecord: \n\t"+singleRecord);	
		
		String reqNO=Utils.genSerialNo();
		System.out.println("3. 得到请求编码 Get the reqNO: "+reqNO);
		
		WSIntDemo intf = new WSIntDemo();
		String rspStr= intf.invokeWSInterface(connConf, reqNO, singleRecord);
	    System.out.println("4. 调用接口获得反馈 Call Interface : The response is:\n\t"+rspStr);
	    
        RspMsg rspMsg=RspMsg.parseRspXML2RspObj(rspStr); 
		if(rspMsg==null){
			System.out.println("5. 调用接口失败 Call interface failed.");
			return;
		}
		System.out.println("5. 得到反馈信息 Get the response:\n\t "+rspMsg.toString());
		
		if(! ("ok".equals(rspMsg.getStatus()))){
			System.out.println("5. Response Error:"+rspMsg.getErrMsg());
			return;
		}				
		System.out.println("**************Demo End**************");
	}

	
	/**
	 * 准备策略输入数据  -
	 * @return
	 */
	private static Map<String, Object> prepareRecord() {
		Map<String, Object> singleRecord = new HashMap<String,Object>();
		singleRecord.put("age", 28);
		singleRecord.put("company", "abc");
		singleRecord.put("appAtNight","Y");
		return singleRecord;
	}
		
	
	/**
	 * 执行决策引擎接口调用
	 * @param connConf 接口调用参数信息
	 * @param reqNO 调用请求编号
	 * @param singleRecord 单条策略输入数据 
	 * @return
	 */
	public String invokeWSInterface(WSConnConf connConf,String reqNO, Map<String, Object> singleRecord) {  
        try {  
            URL url = new URL(WSConnConf.urlStr);  
            URLConnection con = url.openConnection();  
            con.setDoOutput(true);  
            con.setRequestProperty("Pragma", "no-cache");  
            con.setRequestProperty("Cache-Control", "no-cache");  
            con.setRequestProperty("Content-Type", "text/xml");  
  
            
            String reqXML = this.change2Xml(connConf,reqNO, singleRecord);  
            
                         
            System.out.println("---->invokeWSInterface : Starting invoke weekseeker interface...");
            System.out.println("---->invokeWSInterface : The request xml is:"+reqXML);
            OutputStreamWriter out = new OutputStreamWriter(con  
                    .getOutputStream());   
            out.write(reqXML);
            out.flush();  
            out.close();  
            BufferedReader br = new BufferedReader(
            		new InputStreamReader(con.getInputStream(), "utf-8"));  
            String line = "";
            StringBuffer strBuf=new StringBuffer();
            for (line = br.readLine(); line != null; line = br.readLine()) {   
                strBuf.append(line);
            }  
            String rspStr=strBuf.toString();
            //System.out.println("---->invokeWSInterface : The response is:"+rspStr);
            return rspStr;
        } catch (MalformedURLException e) {  
            e.printStackTrace();  
        } catch (IOException e) {  
            e.printStackTrace();  
        }
		return null;  
	}
	
	/**
	 * 将接口调用参数转化成xml格式
	 * @param connConf：注意：
	 * @param serialNo
	 * @param singleRecord 
	 * @return
	 */
	

	private String change2Xml( WSConnConf connConf,String serialNo,
			Map<String, Object> singleRecord){		
		String strategyId = WSConnConf.strategyId;
		String orgId=WSConnConf.orgId;
		String user = WSConnConf.userId;
		String passEncoded=WSConnConf.passEncoded;
		
		Gson gson=new Gson();
		String dataItemJson=gson.toJson(singleRecord);
		dataItemJson=Utils.replaceEscapeSeqs(dataItemJson);
		String result="<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"><soap:Body>"
				+"<ns2:singleDecision xmlns:ns2=\"http://decision.webservice.decision.sdforce.com/\">"
				+"<arg0><orgId>"+orgId+"</orgId><password>"+passEncoded+"</password><serialNo>"
				+serialNo+"</serialNo><strategyId>"+strategyId+"</strategyId><userId>"+user+"</userId><userParams>"
				+dataItemJson+"</userParams></arg0></ns2:singleDecision></soap:Body></soap:Envelope>";	
		return result;
	}
}

